/* Parallel Hierarchical Grid -- an adaptive finite element library.
 *
 * Copyright (C) 2005-2010 State Key Laboratory of Scientific and
 * Engineering Computing, Chinese Academy of Sciences. */

/* This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA  02110-1301  USA */

/* $Id: quad-interface.h,v 1.21 2019/05/17 08:37:49 zlb Exp $
 *
 * Numerical quadrature on an element containing a smooth interface */

#ifndef PHG_QUAD_INTERFACE_H

#ifdef __cplusplus
extern "C" {
#endif

typedef void (*QUAD_1D_FUNC)(FLOAT x, FLOAT *res, const FLOAT wgt, void *ctx);

void phgQuad1D(QUAD_1D_FUNC f, int dim, FLOAT a, FLOAT b, QUAD *quad,
		FLOAT *res, void *ctx);

int phgQuadInterface(DOF *ls, DOF *ls_grad, ELEMENT *e,
		DOF_USER_FUNC func, int dim, DOF_PROJ proj,
		int quad_type, int quad_order, FLOAT *res, FLOAT **prule);

#define phgQuadInterfaceGetRule(ls, ls_grad, e, proj, quad_type, quad_order,\
	prule) \
	phgQuadInterface(ls, ls_grad, e, \
		NULL, (proj) == DOF_PROJ_CROSS ? 3 : 1, proj, \
                quad_type, quad_order, NULL, prule)

int phgQuadInterface2(DOF_USER_FUNC ls, int ls_order, DOF_USER_FUNC ls_grad,
		const FLOAT tet[Dim+1][Dim],
		DOF_USER_FUNC func, int dim, DOF_PROJ proj,
		int quad_type, int quad_order, FLOAT *res, FLOAT **prule);

#define phgQuadInterface2GetRule(ls, ls_order, ls_grad, tet, proj, \
	quad_type, quad_order, prule) \
	phgQuadInterface2(ls, ls_order, ls_grad, tet, \
		NULL, (proj) == DOF_PROJ_CROSS ? 3 : 1, proj, \
                quad_type, quad_order, NULL, prule)

int phgQuadInterfaceApplyRule(DOF_USER_FUNC func, int dim,
		const FLOAT *rule_data, FLOAT *res);

int phgQuadInterfaceDumpRule(const FLOAT *rule_data, FILE *fp);

INT phgQuadInterfaceMarkElements(DOF *ls);

#ifdef __cplusplus
}
#endif

#define PHG_QUAD_INTERFACE_H
#endif
